// -*- C++ -*-
/*!
 * @file OutPortPullConnector.h
 * @brief OutPortPull type connector class
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2009
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

#ifndef RTC_OUTPORTPULLCONNECTOR_H
#define RTC_OUTPORTPULLCONNECTOR_H

#include <rtm/OutPortConnector.h>
#include <rtm/ConnectorListener.h>

namespace RTC
{
  class OutPortProvider;

  /*!
   * @if jp
   * @class OutPortPullConnector
   * @brief OutPortPullConnector NX
   *
   * OutPort  pull ^f[^t[̂߂ Connector NXB̃Iu
   * WFNǵAڑ dataflow_type  pull w肳ꂽꍇA
   * OutPort ɂĐELAInPortPullConnector Ƒ΂ɂȂāA
   * f[^|[g pull ^̃f[^t[B̐ڑɑ΂āA
   * ̃f[^Xg[񋟂B Connector ΉB
   * Connector  ڑɐ UUID ` ID ɂʂB
   *
   * OutPortPullConnector ͈ȉ̎O̃IuWFNgLǗB
   *
   * - InPortConsumer
   * - Buffer
   *
   * OutPort ɏ܂ꂽf[^ OutPortPullConnector::write() ɓn
   *  Buffer ɏ܂BInPortPullConnector 
   * OutPortPullConnector f[^ǂݏoƂ InPort Ƀf[^
   * ]B
   *
   * @since 1.0.0
   *
   * @else
   * @class OutPortPullConnector
   * @brief OutPortPullConnector class
   *
   * Connector class of OutPort for pull type dataflow. When "pull" is
   * specified as dataflow_type at the time of establishing
   * connection, this object is generated and owned by the OutPort.
   * This connector and InPortPullConnector make a pair and realize
   * pull type dataflow of data ports. One connector corresponds to
   * one connection which provides a data stream. Connector is
   * distinguished by ID of the UUID that is generated at establishing
   * connection.
   *
   * OutPortPullConnector owns and manages the following objects.
   *
   * - InPortConsumer
   * - Buffer
   *
   * Data written into the OutPort is passed to
   * OutPortPullConnector::write(), and it is written into the buffer.
   * By reading data from OutPortPullConnector to InPortPullConnector,
   * data transfer is realized.
   *
   * @since 1.0.0
   *
   * @endif
   */
  class OutPortPullConnector
    : public OutPortConnector
  {
  public:
    DATAPORTSTATUS_ENUM

    /*!
     * @if jp
     * @brief RXgN^
     *
     * OutPortPullConnector ̃RXgN^̓IuWFNgɉL
     * ɂƂBConnectorInfo ͐ڑ܂݁Ȁɏ]ob
     * t@𐶐BOutPort C^[tF[X̃voC_IuWFN
     * gւ̃|C^AL̂ŁAOutPortPullConnector 
     * OutPortProvider ̉̐ӔCBeCxgɑ΂R[ob
     * N@\񋟂 ConnectorListeners AK؂ȃ^C~OŃR[
     * obNĂяoBf[^obt@ OutPortBase 񋟂
     * ꍇ͂̃|C^B
     *
     * @param info ConnectorInfo
     * @param provider OutPortProvider
     * @param listeners ConnectorListeners ^̃XiIuWFNgXg
     * @param buffer CdrBufferBase ^̃obt@
     *
     * @else
     * @brief Constructor
     *
     * OutPortPullConnector's constructor is given the following
     * arguments.  According to ConnectorInfo which includes
     * connection information, a buffer is created.  It is also given
     * a pointer to the provider object for the OutPort interface.
     * The owner-ship of the pointer is owned by this
     * OutPortPullConnector, it has responsibility to destruct the
     * OutPortProvider.  OutPortPullConnector also has
     * ConnectorListeners to provide event callback mechanisms, and
     * they would be called at the proper timing.  If data buffer is
     * given by OutPortBase, the pointer to the buffer is also given
     * as arguments.
     *
     * @param info ConnectorInfo
     * @param provider OutPortProvider
     * @param listeners ConnectorListeners type lsitener object list
     * @param buffer CdrBufferBase type buffer
     *
     * @endif
     */
    OutPortPullConnector(ConnectorInfo info,
                         OutPortProvider* provider,
                         ConnectorListeners& listeners,
                         CdrBufferBase* buffer = 0);

    /*!
     * @if jp
     * @brief fXgN^
     *
     * disconnect() Ă΂Aprovider, buffer ́E폜B
     *
     * @else
     *
     * @brief Destructor
     *
     * This operation calls disconnect(), which destructs and deletes
     * the consumer, the publisher and the buffer.
     *
     * @endif
     */
    virtual ~OutPortPullConnector();

    /*!
     * @if jp
     * @brief f[^̏
     *
     * Publisherɑ΂ăf[^݁AɂΉInPort
     * f[^]B
     *
     * @else
     *
     * @brief Writing data
     *
     * This operation writes data into publisher and then the data
     * will be transferred to correspondent InPort.
     *
     * @endif
     */
    virtual ReturnCode write(const cdrMemoryStream& data);

    /*!
     * @if jp
     * @brief ڑ
     *
     * consumer, publisher, buffer ́E폜B
     *
     * @else
     *
     * @brief disconnect
     *
     * This operation destruct and delete the consumer, the publisher
     * and the buffer.
     *
     * @endif
     */
    virtual ReturnCode disconnect();

    /*!
     * @if jp
     * @brief Buffer 擾
     *
     * Connector ێĂ Buffer Ԃ
     *
     * @else
     * @brief Getting Buffer
     *
     * This operation returns this connector's buffer
     *
     * @endif
     */
    virtual CdrBufferBase* getBuffer();

    /*!
     * @if jp
     * @brief ANeBu
     *
     * ̃RlN^ANeBu
     *
     * @else
     *
     * @brief Connector activation
     *
     * This operation activates this connector
     *
     * @endif
     */
    virtual void activate(){}; // do nothing

    /*!
     * @if jp
     * @brief ANeBu
     *
     * ̃RlN^ANeBu
     *
     * @else
     *
     * @brief Connector deactivation
     *
     * This operation deactivates this connector
     *
     * @endif
     */
    virtual void deactivate(){}; // do nothing

    /*!
     * @if jp
     * @brief Buffer̐
     * @else
     * @brief create buffer
     * @endif
     */
    CdrBufferBase* createBuffer(ConnectorInfo& info);

    /*!
     * @if jp
     * @brief ڑmɃR[obNĂ
     * @else
     * @brief Invoke callback when connection is established
     * @endif
     */
    void onConnect();

    /*!
     * @if jp
     * @brief ڑؒfɃR[obNĂ
     * @else
     * @brief Invoke callback when connection is destroied
     * @endif
     */
    void onDisconnect();

  protected:

    /*!
     * @if jp
     * @brief OutPortProvider ւ̃|C^
     * @else
     * @brief the pointer to the OutPortProvider
     * @endif
     */
    OutPortProvider* m_provider;

    /*!
     * @if jp
     * @brief ConnectorListenrs ւ̎Q
     * @else
     * @brief A reference to a ConnectorListener
     * @endif
     */
    ConnectorListeners& m_listeners;

    /*!
     * @if jp
     * @brief Buffer ւ̃|C^
     * @else
     * @brief the pointer to the buffer
     * @endif
     */
    CdrBufferBase* m_buffer;
  };
}; // namespace RTC

#endif  // RTC_PULL_CONNECTOR_H
